{*************
 *
 * Copyright (c) 1996-97 Anton A. Mints
 *
 * Project....: CEO Vusual Library Components
 * File.......: CEOMEdit.pas
 * Autor......: Anton A. Mitns
 * Date.......: 07 January 1997
 * Description: TMaskEdit with AcceptEmpty property. Allows to accept 
 *             empty text value even if mask is set.
 *              Ctrl-Enter combination working ad Dbl-Click.
 *
 *************}

unit CEOMEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask;

type
  TCEOMaskEdit = class(TMaskEdit)
  private
    FFocusedColor: TColor;
    FDefaultColor: TColor;
    FAcceptEmpty: Boolean;

    procedure DoEnter; override;
    procedure DoExit; override;

    procedure CMExit (var Message: TCMExit); message CM_EXIT;

  protected
    procedure KeyPress(var Key: Char); override;
    procedure KeyDown (var Key: Word; Shift: TShiftState); override;

  published
    constructor Create (AOwner: TComponent); override;

    property FocusedColor: TColor read FFocusedColor write FFocusedColor;
    property AcceptEmpty: Boolean read FAcceptEmpty write FAcceptEmpty;
  end;

procedure Register;

implementation

{*******************************************************************}
{*                      CLASS TCEOMaskEdit                         *}
{*******************************************************************}

constructor TCEOMaskEdit.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);

  // V2AAM Assign default color value
  FFocusedColor := Color;
  FDefaultColor := Color;

  // Don't allow to accept empty text
  //if it's masked by default
  FAcceptEmpty := False;
end;


{*************
* Name.......: DoEnter
* Description: Set user defined focused color when control
*             recieve the focuse
*}
procedure TCEOMaskEdit.DoEnter;
begin
  inherited DoEnter;
  Color := FFocusedColor;
end;


{*************
* Name.......: DoExit
* Description: Restore default color when focus are released
*}
procedure TCEOMaskEdit.DoExit;
begin
  inherited DoExit;
  Color := FDefaultColor;
end;


{*************
* Name.......: CMExit (CM_EXIT MESSAGE) / V2AAM
* Description: Don't validate if text is empty
*}
procedure TCEOMaskEdit.CMExit(var Message: TCMExit);
var
  Flag: Boolean;
  CurValue: String;
  CurMask: String;

begin
  CurValue := Text;
  Flag := FAcceptEmpty and
    ( (Text = FormatMaskText (EditMask, '')) or
      (Text = '') );

  if Flag then
    begin
      CurMask := EditMask;
      EditMask := '';
      try
        inherited;
      except
        Text := CurValue;
        raise;
      end;
      EditMask := CurMask;
    end
  else
    inherited;
end;


{*************
* Name.......: KeyPress (OnKeyPress Event)
* Description: Don't validate if text is empty
*}
procedure TCEOMaskEdit.KeyPress(var Key: Char);
var
  KeptEditMask: String;

begin
  // V4AAM Exstensively trim text and resulted mask.
  if ( (Key = #13) and FAcceptEmpty and
    (Trim (Text) = Trim (FormatMaskText (EditMask, ''))) ) then
    try
      KeptEditMask := EditMask;
      EditMask := '';
      inherited KeyPress(Key);
    finally
      EditMask := KeptEditMask;
      Key := #0;
    end
  else
    inherited KeyPress(Key);
end;


{*************
* Name.......: KeyDown (OnKeyPress Event)
* Description: Call DblClick
*}
procedure TCEOMaskEdit.KeyDown (var Key: Word; Shift: TShiftState);
begin
  if ( (ssCtrl in Shift) and (Key = vk_Return) ) then
  begin
    DblClick;
  end;

  inherited KeyDown (Key, Shift);
end;

procedure Register;
begin
  RegisterComponents('CEO', [TCEOMaskEdit]);
end;

end.
